Package: bit
Title: Classes and Methods for Fast Memory-Efficient Boolean Selections
Version: 4.6.0
Authors@R: c(
    person("Michael", "Chirico", email = "MichaelChirico4@gmail.com", role = c("aut", "cre")),
    person("Jens", "Oehlschlägel", role = "aut"),
    person("Brian", "Ripley", role = "ctb")
  )
Depends: R (>= 3.4.0)
Suggests: testthat (>= 3.0.0), roxygen2, knitr, markdown, rmarkdown,
        microbenchmark, bit64 (>= 4.0.0), ff (>= 4.0.0)
Description: Provided are classes for boolean and skewed boolean vectors,
    fast boolean methods, fast unique and non-unique integer sorting,
    fast set operations on sorted and unsorted sets of integers, and
    foundations for ff (range index, compression, chunked processing).
License: GPL-2 | GPL-3
LazyLoad: yes
ByteCompile: yes
Encoding: UTF-8
URL: https://github.com/r-lib/bit
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-03-05 07:18:45 UTC; michael
Author: Michael Chirico [aut, cre],
  Jens Oehlschlägel [aut],
  Brian Ripley [ctb]
Maintainer: Michael Chirico <MichaelChirico4@gmail.com>
Repository: CRAN
Date/Publication: 2025-03-06 10:50:01 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 08:59:55 UTC; unix
Archs: bit.so.dSYM
