Package: bit
Type: Package
Title: Classes and Methods for Fast Memory-Efficient Boolean Selections
Version: 4.0.5
Date: 2022-11-13
Author: Jens Oehlschlägel [aut, cre], Brian Ripley [ctb]
Maintainer: Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
Depends: R (>= 2.9.2)
Suggests: testthat (>= 0.11.0), roxygen2, knitr, rmarkdown,
        microbenchmark, bit64 (>= 4.0.0), ff (>= 4.0.0)
Description: Provided are classes for boolean and skewed boolean vectors,
    fast boolean methods, fast unique and non-unique integer sorting,
    fast set operations on sorted and unsorted sets of integers, and
    foundations for ff (range index, compression, chunked processing).
License: GPL-2 | GPL-3
LazyLoad: yes
ByteCompile: yes
Encoding: UTF-8
URL: https://github.com/truecluster/bit
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 7.2.0
NeedsCompilation: yes
Packaged: 2022-11-13 21:22:09 UTC; jo
Repository: CRAN
Date/Publication: 2022-11-15 21:20:16 UTC
