% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integerutil.R
\name{reverse_vector}
\alias{reverse_vector}
\title{Reverse atomic vector}
\usage{
reverse_vector(x)
}
\arguments{
\item{x}{an R vector}
}
\value{
a reversed vector
}
\description{
Returns a reversed copy -- with attributes retained.
}
\details{
This is substantially faster than \code{\link{rev}}
}
\examples{
x <- factor(letters)
rev(x)
reverse_vector(x)
\dontrun{
x <- 1:1e7
system.time(rev(x))
system.time(reverse_vector(x))
}
}
\seealso{
\code{\link{rev}}, \code{\link{copy_vector}}
}
